// blackhole warp shader for wormholes (Radius <= 1.0) ; Gargantua (Radius >= 3 Au) and normal black holes (Radius > 1.0  < 3 Au)

#version 330 core
#auto_defines

uniform sampler2D   Scene;
uniform vec4        PosScale;   // (BH x, BH y, BH scale x, BH scale y)
uniform vec3        RadDist;    // (BH radius, distance, 2*radius/distance)

#ifdef _VERTEX_

layout(location = 0) in  vec4  VertexPos;
layout(location = 1) in  vec4  VertexTexCoord;
                     out vec2  TexCoord;

void main()
{
    gl_Position = VertexPos;
    TexCoord = VertexTexCoord.xy;
}

#else

                     in  vec2  TexCoord;
layout(location = 0) out vec4  OutColor;

void main()
{
	vec2  texCoord = TexCoord - PosScale.xy;
	float rad = length(texCoord * PosScale.zw);

	if (RadDist.x > 3.0 * 149597870.691)   // Gargantua black hole  > 3 AU
	{
		float distort = max(RadDist.z / (rad * 1.6), 0.0);
		texCoord = texCoord * (1.0 - distort) + PosScale.xy;
		//texCoord = clamp(texCoord, 0.0, 1.0);
		OutColor = texture(Scene, texCoord);
		OutColor *= clamp((rad * RadDist.y - RadDist.x) / RadDist.x * 10.0, 0.0, 1.0); // antialiased event horizon
		}
	else if (RadDist.x > 1.0) 		// normal black hole
	{
		float distort = max(RadDist.z / (rad * rad), 0.0);
		texCoord = texCoord * (1.0 - distort) + PosScale.xy;
		//texCoord = clamp(texCoord, 0.0, 1.0);
		OutColor = texture(Scene, texCoord);
		OutColor *= clamp((rad * RadDist.y - RadDist.x) / RadDist.x * 100.0, 0.0, 1.0); // antialiased event horizon
	}
	else    // black hole <= (Radius = 1.0)
	{
		float rad = length(texCoord * PosScale.zw * 0.42);
		float distort = max(RadDist.z - (rad), 0.0);
		texCoord = texCoord / (1.0 - distort * 250) + PosScale.xy;
		OutColor = texture(Scene, texCoord);
		OutColor *= clamp((rad * RadDist.y - RadDist.x) / RadDist.x * 10.0, 1.0, 1.0); // antialiased event horizon
	}
}

#endif
